@@ -7,6 +7,7 @@ from functools import lru_cache
 from urllib.parse import urlparse, urlunparse
 import re
 import logging
+import socket
 
 import requests
 import requests.utils
@@ -129,6 +130,16 @@ class UnrelatedRepository(error):
 class RateLimitExceeded(error):
     reason = "Site rate limit exceeded. Too many requests!"
 
+class NetworkError(error):
+    pass
+
+
+def http_get(*args, **kargs):
+    try:
+        return requests.get(*args, **kargs)
+    except requests.exceptions.ConnectionError as e:
+        raise NetworkError(e)
+
 
 class WorkspaceName:
     def __init__(self, full_workspace, site=None):
@@ -158,7 +169,7 @@ class WorkspaceName:
 
 class RepoRef:
     def __init__(self, site_full_name, site=None):
-        if site_full_name.startswith(('https://', 'git@')):
+        if site_full_name.startswith(('https://', 'git@', 'ssh://')):
             site_full_name = RepoRef.parse_origin(site_full_name)
 
         if site_full_name.count('/') != 1:
@@ -176,16 +187,19 @@ class RepoRef:
 
     @classmethod
     def parse_origin(cls, url):
-        if url.startswith('git@'):
+        if url.startswith('ssh://'):
+            fields = re.findall(r'\Assh://git@([^/]+)/(.+).git\Z', url)[0]
+
+        elif url.startswith('git@'):
             fields = re.findall(r'\Agit@([^:]+):(.+).git\Z', url)[0]
-            return '{}:{}'.format(sites[fields[0]], fields[1])
 
         elif url.startswith('https://'):
-            print(url)
             fields = re.findall(r'\Ahttps?://([^/]+)/(.+)(?:\.git)?\Z', url)[0]
-            return '{}:{}'.format(sites[fields[0]], fields[1])
 
-        raise BadRepositoryName(url)
+        else:
+            raise BadRepositoryName(url)
+
+        return '{}:{}'.format(sites[fields[0]], fields[1])
 
     @classmethod
     def from_origin(cls, url):
@@ -240,15 +254,13 @@ class Completion:
 
         # for site:name names
         if name.count(':') == 1:
-            print(name)
             site, name = name.split(':')
             workspaces = [ws for ws in workspaces if ws.site == SITE_ABBREVS[site]]
-            print(workspaces)
 
         for ws in workspaces:
             try:
                 repo = ws.make_repo(name)
-                if repo.exists():
+                if repo.checked_exists():
                     self.found.append(repo.ref.global_name)
             except AccessDenied:
                 self.denied.append(repo.ref.global_name)
@@ -391,6 +403,7 @@ def safe_url(url):
 
 class Repo(Auth):
     def check(self):
+        "check access to server"
         self.webpage
         return True
 
@@ -401,11 +414,24 @@ class Repo(Auth):
         except RepositoryNotFound:
             return False
 
+    def checked_exists(self):
+        "Renamed repos keep old URLs, so must verify same slug"
+        try:
+            if not self.exists():
+                return False
+
+            self._load_full_data()
+            return self.slug == self.name
+        except RepositoryNotFound:
+            pass
+
+        return False
+
     def _load_full_data(self):
         logging.debug(self.url)
 
         # FIXME: catch connection exceptions
-        result = requests.get(self.url)
+        result = http_get(self.url)
         self.reply_check(result)
         retval = result.json()
         retval['access'] = self._get_access(retval)
@@ -423,7 +449,6 @@ class Repo(Auth):
             except KeyError:
                 raise AttributeError(attr)
 
-
     @classmethod
     def from_dir(cls, dirname, credentials=None):
         try:
@@ -623,7 +648,7 @@ class BitbucketRepo(Repo):
 
     def last_commits(self, max_=3):
         commits_url = self.url + '/commits'
-        result = requests.get(commits_url)
+        result = http_get(commits_url)
         self.reply_check(result)
         commits = result.json()['values']
 
@@ -662,7 +687,7 @@ class BitbucketRepo(Repo):
         URL = 'https://api.bitbucket.org/2.0/user/permissions/repositories?q=repository.name="{}"'
         url = self.auth(URL.format(self.ref.slug))
         print(url)
-        result = requests.get(url)
+        result = http_get(url)
         print(self.ref)
         self.reply_check(result)
         result = result.json()
@@ -739,7 +764,7 @@ class GithubRepo(Repo):
         url = self.url + '/commits'
         logging.debug(url)
 
-        result = requests.get(url)
+        result = http_get(url)
         self.reply_check(result, [200, 409])
         if result.status_code == 409:
             return []
@@ -789,9 +814,6 @@ class GithubRepo(Repo):
             self.url,
             data=json.dumps({'name': new_name}))
 
-        print(result.status_code)
-        print(result.json())
-
         self.reply_check(result)
         real_name = result.json()['name'].split('/')[-1]
         return real_name
@@ -821,7 +843,7 @@ class BitbucketWorkspace(Workspace):
         next_link = self.url
         while next_link is not None:
             next_link = self.auth(next_link)
-            result = requests.get(next_link)
+            result = http_get(next_link)
             logging.debug(next_link)
             Bitbucket.api_check(result)
 
@@ -831,7 +853,7 @@ class BitbucketWorkspace(Workspace):
                 yield BitbucketRepo.from_data(repo, self.credentials)
 
     def check(self):
-        Bitbucket.api_check(requests.get(self.auth(self.url)))
+        Bitbucket.api_check(http_get(self.auth(self.url)))
 
     def make_repo(self, reponame):
         return BitbucketRepo(
@@ -852,7 +874,7 @@ class GithubWorkspace(Workspace):
 
         self.name = name
         self.url = self.ORG_URL.format(org=name.workspace)
-        result = requests.get(self.auth(self.url))
+        result = http_get(self.auth(self.url))
         if result.status_code == 404:
             logging.info("'{}' is not an organization. Trying as user.".format(
                 name.workspace))
@@ -875,7 +897,7 @@ class GithubWorkspace(Workspace):
         next_link = self.url
         while next_link is not None:
             next_link = self.auth(next_link)
-            result = requests.get(next_link)
+            result = http_get(next_link)
             logging.debug(next_link)
             Github.api_check(result)
 
@@ -886,7 +908,7 @@ class GithubWorkspace(Workspace):
                 yield GithubRepo.from_data(repo, self.credentials)
 
     def check(self):
-        Github.api_check(requests.get(self.auth(self.url)))
+        Github.api_check(http_get(self.auth(self.url)))
 
     def make_repo(self, reponame):
         return GithubRepo(
