@@ -6,10 +6,13 @@ import argparse
 import logging
 from pathlib import Path
 import webbrowser
+import warnings
 
 import ripio
 import ripio.utils as utils
 
+warnings.filterwarnings('ignore')
+
 
 def set_verbosity(args):
     level = logging.ERROR - 10 * min(args.verbosity, 3)
